from web3 import Web3, middleware
from web3.gas_strategies.time_based import medium_gas_price_strategy

class Providers:
    def __init__(self, l1, l2, testnet:bool):
        self.testnet = testnet
        self.l1 = l1 
        self.l2 = l2
        #self.l1.eth.set_gas_price_strategy(medium_gas_price_strategy)
        #self.l2.eth.set_gas_price_strategy(medium_gas_price_strategy)
        #self.l1.middleware_onion.add(middleware.time_based_cache_middleware)
        #self.l1.middleware_onion.add(middleware.latest_block_based_cache_middleware)
        #self.l1.middleware_onion.add(middleware.simple_cache_middleware)
        #self.l2.middleware_onion.add(middleware.time_based_cache_middleware)
        #self.l2.middleware_onion.add(middleware.latest_block_based_cache_middleware)
        #self.l2.middleware_onion.add(middleware.simple_cache_middleware)


import json
from pyperp import MetaData
from pyperp.utils import estimatedFundingRate, formatUnits
import pkgutil

def getAmmInfo(provider,pair=None):
    insuranceFundAbi = json.loads(pkgutil.get_data(__name__,"abi/InsuranceFund.json"))
    clearingHouseAbi = json.loads(pkgutil.get_data(__name__,"abi/ClearingHouse.json"))
    ammAbi = json.loads(pkgutil.get_data(__name__,"abi/Amm.json"))

    meta = MetaData.MetaData(provider.testnet)
    insuranceFundAddr = meta.getL2ContractAddress("InsuranceFund")
    clearingHouseAddr = meta.getL2ContractAddress("ClearingHouse")

    insuranceFund = provider.l2.eth.contract(address=insuranceFundAddr, abi=insuranceFundAbi)
    clearingHouse = provider.l2.eth.contract(address=clearingHouseAddr, abi=clearingHouseAbi)

    data = []

    ammAddressList = insuranceFund.functions.getAllAmms().call()

    for addr in ammAddressList:
        amm = provider.l2.eth.contract(address=addr, abi=ammAbi)
        print(ammAbi)
        priceFeedKey = amm.functions.priceFeedKey().call().decode("utf-8","ignore")
        if pair is not None and pair != priceFeedKey:
            continue
        
        openInterestNotionalCap = amm.functions.getOpenInterestNotionalCap().call()
        openInterestNotional = clearingHouse.functions.openInterestNotinalMap(addr).call()
        maxHoldingBaseAsset = amm.functions.getMaxHoldingBaseAsset().call()
        indexPrice = amm.functions.getUnderlyingPrice().call()
        marketPrice = amm.functions.getSpotPrice().call()
        reserve = amm.functions.getReserve().call()
        quoteAssetAddress = amm.functions.quoteAsset().call()
        quoteAssetReserve = reserve[0]
        baseAssetReserve = reserve[1]
        priceFeed = amm.functions.priceFeed().call()
        estFundingRate = (estimatedFundingRate(amm)/1e18) * 100
        
        if priceFeed == meta.getL2ContractAddress("L2PriceFeed"):
            priceFeedName = "L2PriceFeed"
        elif priceFeed == meta.getL2ContractAddress("ChainlinkPriceFeed"):
            priceFeedName = "ChainlinkPriceFeed"
        else:
            ValueError("PriceFeed is not L2PriceFeed or ChainlinkPriceFeed, check it immediately!! address: " + priceFeed)

        data.append({
            "pair":f"{priceFeedKey}/USDC",
            "Proxy Address": addr,
            "Index Price": f"{formatUnits(indexPrice)} USDC",
            "Market Price": f"{formatUnits(marketPrice)} USDC",
            "OpenInterestNotionalCap": f"{formatUnits(openInterestNotionalCap)} USDC",
            "OpenInterestNotional": f"{formatUnits(openInterestNotinal)} USDC",
            "MaxHoldingBaseAsset": f"{formatUnits(maxHoldingBaseAsset)} USDC",
            "QuoteAssetReserve": f"{formatUnits(quoteAssetReserve)} USDC",
            "BaseAssetReserve": f"{formatUnits(baseAssetReserve)} {priceFeedKey}USDC",
            "PriceFeed": priceFeedName,
            "est.funding rate": f"{estFundingRate} %"
        })

    return data


from pyperp import constants
import json
from pyperp.MetaData import MetaData
from pyperp.constants import Side
from decimal import Decimal 
from pyperp.utils import *
from eth_abi import encode_single
import pkgutil

class Trader:
    def __init__(self,provider,l1wallet,l2wallet):
        self._layer1wallet = l1wallet
        self._layer2wallet = l2wallet
        self._provider = provider
        self.meta = MetaData.MetaData(provider.testnet)

        clearingHouseAddr = self.meta.getL2ContractAddress("ClearingHouse")
        clearingHouseAbi =  json.loads(pkgutil.get_data(__name__,"abi/ClearingHouse.json"))
        self.clearingHouse = self._provider.l2.eth.contract(address=clearingHouseAddr, abi=clearingHouseAbi)

        clearingHouseViewerAddr = self.meta.getL2ContractAddress("ClearingHouseViewer")
        clearingHouseViewerAbi = json.loads(pkgutil.get_data(__name__,"abi/ClearingHouseViewer.json"))
        self.clearingHouseViewer = self._provider.l2.eth.contract(address=clearingHouseViewerAddr, abi=clearingHouseViewerAbi)


    @property
    def layer1wallet(self):
        return self._layer1wallet

    @layer1wallet.setter
    def layer1wallet(self, w):
        self._layer1wallet = w

    @property
    def layer2wallet(self):
        return self._layer2wallet

    @layer2wallet.setter
    def layer2wallet(self, w):
        self._layer2wallet = w

    def l1WalletBalance(self):
        return self._provider.l1.eth.getBalance(self._layer1wallet.address)

    def l2WalletBalance(self):
        return self._provider.l2.eth.getBalance(self._layer2wallet.address)

    def approveL1BridgetoUseUSDC(self):
        
        TetherTokenAbi = json.loads(pkgutil.get_data(__name__,"abi/TetherToken.json"))
        UsdcAddr = self.meta.getL1ExtContractAddress("usdc")
        layer1BridgeAddr = self.meta.getL1ContractAddress("RootBridge")
        layer1Usdc = self._provider.l1.eth.contract(address=UsdcAddr, abi=TetherTokenAbi)
        nonce = self._provider.l1.eth.get_transaction_count(self._layer1wallet.address)
        approveTx = layer1Usdc.functions.approve(layer1BridgeAddr, constants.MaxUInt256).buildTransaction({
            'nonce':nonce,
            'gas': 1000000,
            'gasPrice': self._provider.l1.eth.gasPrice,
        })
        signed_tx = self._provider.l1.eth.account.sign_transaction(approveTx, private_key=self._layer1wallet.key)
        
        approveTxHash = self._provider.l1.eth.send_raw_transaction(signed_tx.rawTransaction)
        receipt = self._provider.l1.eth.wait_for_transaction_receipt(approveTxHash)
        return receipt

    def depositUsdcToxDai(self, amount):
        TetherTokenAbi = json.loads(pkgutil.get_data(__name__,"abi/TetherToken.json"))
        RootBridgeAbi = json.loads(pkgutil.get_data(__name__,"abi/RootBridge.json"))
        UsdcAddr = self.meta.getL1ExtContractAddress("usdc")
        layer1BridgeAddr = self.meta.getL1ContractAddress("RootBridge")
        layer1Usdc = self._provider.l1.eth.contract(address=UsdcAddr, abi=TetherTokenAbi)
        layer1Bridge = self._provider.l1.eth.contract(address=layer1BridgeAddr, abi=RootBridgeAbi)
        nonce = self._provider.l1.eth.get_transaction_count(self._layer1wallet.address)
        #gasEstimate = layer1Bridge.functions.erc20Transfer(UsdcAddr, self._layer1wallet.address, {'d':parseUnits(amount,18)}).estimateGas() 
        transferTx = layer1Bridge.functions.erc20Transfer(UsdcAddr, self._layer1wallet.address, {'d':parseUnits(amount,18)}).buildTransaction({
            'nonce':nonce,
            'gas': 1000000,
            'gasPrice': self._provider.l1.eth.gasPrice,
        })
        signed_tx = self._provider.l1.eth.account.sign_transaction(transferTx, private_key=self._layer1wallet.key)
        transferTxHash = self._provider.l1.eth.send_raw_transaction(signed_tx.rawTransaction)
        receipt = self._provider.l1.eth.wait_for_transaction_receipt(transferTxHash)
        return receipt
        
    def approveL2BridgeToUseUSDC(self):
        TetherTokenAbi = json.loads(pkgutil.get_data(__name__,"abi/TetherToken.json"))
        UsdcAddr = self.meta.getL2ExtContractAddress("usdc")
        layer2BridgeAddr = self.meta.getL2ContractAddress("ClientBridge")
        layer2Usdc = self._provider.l2.eth.contract(address=UsdcAddr, abi=TetherTokenAbi)
        nonce = self._provider.l2.eth.get_transaction_count(self._layer2wallet.address)
        #gasEstimate = layer2Usdc.functions.approve(layer2BridgeAddr, constants.MaxUInt256).estimateGas()
        approveTx = layer2Usdc.functions.approve(layer2BridgeAddr, constants.MaxUInt256).buildTransaction({
            'nonce':nonce,
            'gas': 1000000,
            'gasPrice': self._provider.l2.eth.gasPrice,
        })
        signed_tx = self._provider.l2.eth.account.sign_transaction(approveTx, private_key=self._layer2wallet.key)
        
        approveTxHash = self._provider.l2.eth.send_raw_transaction(signed_tx.rawTransaction)
        receipt = self._provider.l2.eth.wait_for_transaction_receipt(approveTxHash)
        return receipt
    
    def withdrawUsdcToEthereum(self, amount):
        
        TetherTokenAbi = json.loads(pkgutil.get_data(__name__,"abi/TetherToken.json"))
        ClientBridgeAbi = json.loads(pkgutil.get_data(__name__,"abi/ClientBridge.json"))
        UsdcAddr = self.meta.getL2ExtContractAddress("usdc")
        layer2BridgeAddr = self.meta.getL2ContractAddress("ClientBridge")
        layer2Usdc = self._provider.l2.eth.contract(address=UsdcAddr, abi=TetherTokenAbi)
        layer2Bridge = self._provider.l2.eth.contract(address=layer2BridgeAddr, abi=ClientBridgeAbi)
        nonce = self._provider.l2.eth.get_transaction_count(self._layer2wallet.address)
        #gasEstimate = layer2Bridge.functions.erc20Transfer(layer2Usdc.address, self._layer2wallet.address,{'d':parseUnits(amount,constants.DEFAULT_DECIMALS)}).esimateGas()
        transferTx = layer2Bridge.functions.erc20Transfer(layer2Usdc.address, self._layer2wallet.address,{'d':parseUnits(amount,constants.DEFAULT_DECIMALS)}).buildTransaction({
            'nonce':nonce,
            'gas': 1000000,
            'gasPrice':self._provider.l2.eth.gasPrice
        })
        signed_tx = self._provider.l2.eth.account.sign_transaction(transferTx, private_key=self._layer2wallet.key)
        transferTxHash = self._provider.l2.eth.send_raw_transaction(signed_tx.rawTransaction)
        receipt = self._provider.l2.eth.wait_for_transaction_receipt(transferTxHash)
        return receipt

    def approveClearingHouseToUseUSDC(self):
        
        TetherTokenAbi = json.loads(pkgutil.get_data(__name__,"abi/TetherToken.json"))
        UsdcAddr = self.meta.getL2ExtContractAddress("usdc")
        ClearingHouseAddr = self.meta.getL2ContractAddress("ClearingHouse")
        layer2Usdc = self._provider.l2.eth.contract(address=UsdcAddr, abi=TetherTokenAbi)
        nonce = self._provider.l2.eth.get_transaction_count(self._layer2wallet.address)
        #gasEstimate = layer2Usdc.functions.approve(ClearingHouseAddr, constants.MaxUInt256).estimateGas()
        tx = layer2Usdc.functions.approve(ClearingHouseAddr, constants.MaxUInt256).buildTransaction({
            'nonce':nonce,
            'gas': 1000000,
            'gasPrice':self._provider.l2.eth.gasPrice
        })
        signed_tx = self._provider.l2.eth.account.sign_transaction(tx,private_key=self._layer2wallet.key)
        txHash = self._provider.l2.eth.send_raw_transaction(signed_tx.rawTransaction)
        receipt = self._provider.l2.eth.wait_for_transaction_receipt(txHash)
        return receipt
    
    def openPosition(self, pair, side, quoteAssetAmount, leverage, baseAssetAmountLimit):
        
        if side!=0 and side!=1:
            raise ValueError("side must be either 0 or 1")

        if quoteAssetAmount <= 0:
            raise ValueError("quoteAssetAmount must be greater than 0")

        if leverage <= 0 or leverage > 10:
            raise ValueError("leverage must be in the range (0,10]")

        Amm = getAmm(pair,self._provider)

        nonce = self._provider.l2.eth.get_transaction_count(self._layer2wallet.address)
        #gasEstimate = self.clearingHouse.functions.openPosition(Amm.address, side, {'d': parseUnits(quoteAssetAmount,18)}, {'d':parseUnits(leverage,18)}, {'d':parseUnits(baseAssetAmountLimit,18)}).estimateGas() 
        tx = self.clearingHouse.functions.openPosition(Amm.address, side, {'d': parseUnits(quoteAssetAmount,18)}, {'d':parseUnits(leverage,18)}, {'d':parseUnits(baseAssetAmountLimit,18)}).buildTransaction({
            'nonce':nonce,
            'gas': 1000000,
            'gasPrice':self._provider.l2.eth.gasPrice
        })
        signed_tx = self._provider.l2.eth.account.sign_transaction(tx,private_key=self._layer2wallet.key)
        txHash = self._provider.l2.eth.send_raw_transaction(signed_tx.rawTransaction)
        receipt = self._provider.l2.eth.wait_for_transaction_receipt(txHash)
        return receipt

    def closePosition(self, pair, quoteAssetAmountLimit):
        
        Amm = getAmm(pair,self._provider)
        
        nonce = self._provider.l2.eth.get_transaction_count(self._layer2wallet.address)
        #gasEstimate = self.clearingHouse.functions.closePosition(Amm.address, {"d":parseUnits(quoteAssetAmountLimit,constants.DEFAULT_DECIMALS)}).estimateGas()
        tx  = self.clearingHouse.functions.closePosition(Amm.address, {"d":parseUnits(quoteAssetAmountLimit,constants.DEFAULT_DECIMALS)}).buildTransaction({
            'nonce':nonce,
            'gas': 300000,
            'gasPrice':self._provider.l2.eth.gasPrice
        })
        signed_tx = self._provider.l2.eth.account.sign_transaction(tx,private_key=self._layer2wallet.key)
        txHash = self._provider.l2.eth.send_raw_transaction(signed_tx.rawTransaction)
        receipt = self._provider.l2.eth.wait_for_transaction_receipt(txHash)
        return receipt

    def getPersonalPositionWithFundingPayment(self, pair, trader=None):
        #note: return for all open positions
        Amm = getAmm(pair,self._provider)

        if trader is None:
            trader = self._layer2wallet.address

        position = self.clearingHouseViewer.functions.getPersonalPositionWithFundingPayment(Amm.address,trader).call()        
        return {
            'size': position[0][0],
            'margin': position[1][0],
            'openNotional': position[2][0],
            'lastUpdatedCumulativePremiumFraction': position[3][0],
            'liquidityHistoryIndex': position[4],
            'blockNumber': position[5]
        }

    def getUnrealizedPnl(self, pair, pnlCalcOption, trader = None):
        #note: return for all open positions
        Amm = getAmm(pair,self._provider)
        if trader is None:
            trader = self._layer2wallet.address

        return self.clearingHouseViewer.functions.getUnrealizedPnl(Amm.address,trader,pnlCalcOption.value).call()[0]

    def getEntryPrice(self, pair):

        position = self.getPersonalPositionWithFundingPayment(pair)
        openNotional = formatUnits(position["openNotional"], constants.DEFAULT_DECIMALS)
        size  = formatUnits(position["size"], constants.DEFAULT_DECIMALS)

        entryPrice = abs(openNotional/size)
        return entryPrice

    def addMargin(self, pair, margin):

        Amm = getAmm(pair,self._provider)

        nonce = self._provider.l2.eth.get_transaction_count(self._layer2wallet.address)
        #gasEstimate = self.clearingHouse.functions.addMargin(Amm.address,{'d':parseUnits(margin,18)}).estimateGas()
        tx = self.clearingHouse.functions.addMargin(Amm.address,{'d':parseUnits(margin,18)}).buildTransaction({
            'nonce':nonce,
            'gas': 300000,
            'gasPrice':self._provider.l2.eth.gasPrice
        })
        signed_tx = self._provider.l2.eth.account.sign_transaction(tx,private_key=self._layer2wallet.key)
        txHash = self._provider.l2.eth.send_raw_transaction(signed_tx.rawTransaction)
        receipt = self._provider.l2.eth.wait_for_transaction_receipt(txHash)
        return receipt

    def removeMargin(self, pair, margin):

        Amm = getAmm(pair,self._provider)

        nonce = self._provider.l2.eth.get_transaction_count(self._layer2wallet.address)
        #gasEstimate = self.clearingHouse.functions.removeMargin(Amm.address,{'d':parseUnits(margin,18)}).estimateGas()
        tx = self.clearingHouse.functions.removeMargin(Amm.address,{'d':parseUnits(margin,18)}).buildTransaction({
            'nonce':nonce,
            'gas': 300000,
            'gasPrice':self._provider.l2.eth.gasPrice
        })
        signed_tx = self._provider.l2.eth.account.sign_transaction(tx,private_key=self._layer2wallet.key)
        txHash = self._provider.l2.eth.send_raw_transaction(signed_tx.rawTransaction)
        receipt = self._provider.l2.eth.wait_for_transaction_receipt(txHash)
        return receipt

    def settlePosition(self, pair):

        Amm = getAmm(pair, self._provider)

        nonce = self._provider.l2.eth.get_transaction_count(self._layer2wallet.address)
        #gasEstimate = self.clearingHouse.functions.removeMargin(Amm.address).estimateGas()
        tx = self.clearingHouse.functions.settlePosition(Amm.address).buildTransaction({
            'nonce':nonce,
            'gas': 300000,
            'gasPrice':self._provider.l2.eth.gasPrice
        })
        signed_tx = self._provider.l2.eth.account.sign_transaction(tx,private_key=self._layer2wallet.key)
        txHash = self._provider.l2.eth.send_raw_transaction(signed_tx.rawTransaction)
        receipt = self._provider.l2.eth.wait_for_transaction_receipt(txHash)
        return receipt

    def liquidate(self, pair):

        Amm = getAmm(pair, self._provider)
        
        nonce = self._provider.l2.eth.get_transaction_count(self._layer2wallet.address)
        #gasEstimate = self.clearingHouse.functions.removeMargin(Amm.address, self._layer2wallet.address).estimateGas()
        tx = self.clearingHouse.functions.liquidate(Amm.address, self._layer2wallet.address).buildTransaction({
            'nonce':nonce,
            'gas': 300000,
            'gasPrice':self._provider.l2.eth.gasPrice
        })
        signed_tx = self._provider.l2.eth.account.sign_transaction(tx,private_key=self._layer2wallet.key)
        txHash = self._provider.l2.eth.send_raw_transaction(signed_tx.rawTransaction)
        receipt = self._provider.l2.eth.wait_for_transaction_receipt                           (txHash)
        return receipt

    



from pyperp.Trader import (
    Trader
)

from pyperp.Providers import (
    Providers
)

from pyperp.MetaData import (
    MetaData
)

import pyperp.Faucets
import pyperp.constants
import pyperp.utils


__all__ = [
    "constants",
    "Faucets",
    "MetaData",
    "Providers",
    "Trader",
    "utils"
]


import requests
import json 

def getUsdcL1(address):
    faucetApiKey = "da2-h4xlnj33zvfnheevfgaw7datae"
    appSyncId = "izc32tpa5ndllmbql57pcxluua"
    faucet = f"https://{appSyncId}.appsync-api.ap-northeast-1.amazonaws.com/graphql"
    headers = {
        "Content-Type": "application/json",
        "X-Api-Key": faucetApiKey
    }
    body = {
        'query':f'mutation issue {{issue(holderAddr: "{address}"){{\
                txHashQuote\
                amountQuote\
            }}\
        }}'
    }
    r = requests.post(faucet, data=json.dumps(body), headers=headers)
    return r 



import json
from pyperp import MetaData, constants
import datetime 
import pkgutil 

def confirmTransferToLayer2(receipt, provider):
    meta = MetaData(provider.testnet)
    layer2AmbAbi = json.loads(pkgutil.get_data(__name__,"abi/AmbL2.json"))
    layer2AmbAddr = meta.getL2ExtContractAddress("ambBrideOnXDai")
    layer2Amb =  provider.l2.eth.contract(address=layer2AmbAddr,abi=layer2AmbAbi)
    methodId = "0x482515ce"
    for r in receipt:
        log = r["logs"]
        if log.topics[0][:10]==methodId:
            break
    fromMsgId = log.topics[1]
    confirmationFilter = layer2Amb.events.AffirmationCompleted.createFilter(fromBlock="latest",argument_filters={'toMsgId':fromMsgId})  
    while True:
        for c in confirmationFilter:
            print("Transfer completed")
            return 

def confirmTransferToLayer1(reciept, provider):
    meta = MetaData(provider.testnet)
    layer1AmbAbi = json.loads(pkgutil.get_data(__name__,"abi/AmbL1.json"))
    lauer1AmbAddr = meta.getL1ExtContractAddress("ambBridgeOnEth")
    layer1Amb = provider.l1.eth.contract(address=layer1AmbAddr,abi=layer1AmbAbi)
    methodId = "0x520d2afd"
    for r in receipt:
        log = r["logs"]
        if log.topics[0][:10]==methodId:
            break
    fromMsgId = log.topics[1]
    confirmationFilter = layer1Amb.events.RelayedMessage.createFilter(fromBlock="latest",argument_filters={'toMsgId':fromMsgId})  
    while True:
        for c in confirmationFilter:
            print("Transfer completed")
            return 

def getAmm(amm, provider):
    if provider.testnet:
            Amms = constants.AvailableAmms["testnet"]
    else:
        Amms = constants.AvailableAmms["mainnet"]

    if amm not in Amms:
        raise ValueError(f"Unknown pair: {pair}")

    meta = MetaData.MetaData(provider.testnet)
    AmmAddr = meta.getL2ContractAddress(amm)
    AmmAbi = json.loads(pkgutil.get_data(__name__,"abi/Amm.json"))

    Amm = provider.l2.eth.contract(address=AmmAddr, abi=AmmAbi)
    return Amm

def estimateLiquidationPrice(position, amm, clearingHouse):
    spotPrice = amm.functions.getSpotPricer.call().d
    realCloseQuoteAmount = amm.functions.getOutputPrice(
        0 if position.size.d > 0 else 1,
        json.dumps(f"d: {abs(position.size.d)}")
    ).call().d
    maintanenceMarginRatio = clearingHouse.functions.maintanenceMarginRatio().call()
    openNotional = position.openNotional.d
    positionSizeAbs = abs(position.size.d)
    margin = position.margin.d
    entryPrice = openNotional/positionSizeAbs
    reverseLeverage = margin/openNotional
    spotCloseQuoteAmount = spotPrice*positionSizeAbs
    closePosPriceSlippage = (realCloseQuoteAmount - spotCloseQuoteAmount)/spotCloseQuoteAmount
    if position.size.d > 0:
        liquidationPrice =  entryPrice*(1-reverseLeverage-closePosPriceSlippage+maintanenceMarginRatio)
    else:
        liquidationPrice = entryPrice*(1+reverseLeverage-closePosPriceSlippage-maintanenceMarginRatio)
    return liquidationPrice

def parseUnits(amount, decimal=constants.DEFAULT_DECIMALS):
    return amount*(10**decimal)

def formatUnits(amount, decimal=constants.DEFAULT_DECIMALS):
    return amount/(10**decimal)

def estimatedFundingRate(amm):
    durationFromSharp = DateTime.local().minute * 60
    twapPrice = amm.functions.getTwapPrice(durationFromSharp).call()
    underlyingTwapPrice = amm.functions.getUnderlyingTwapPrice(durationFromSharp).call()
    fundingPeriod = amm.functions.fundingPeriod().call()
    
    oneDayInSec = 60*60*24
    marketTwapPrice = formatUnits(twapPrice[0])
    indexTwapPrice = formatUnits(underlyingTwapPrice[0])
    premium = marketTwapPrice - indexTwapPrice
    premiumFraction = premium*fundingPeriod/oneDayInSec
    return parseUnits(premiumFraction/indexTwapPrice)



from enum import Enum 

MaxUInt256 = 2**256-1

AvailableAmms = {
    "mainnet":[
        "ETHUSDC","BTCUSDC","YFIUSDC",
        "DOTUSDC","SNXUSDC","LINKUSDC",
        "AAVEUSDC","SUSHIUSDC","COMPUSDC",
        "RENUSDC","PERPUSDC","UNIUSDC",
        "CRVUSDC","MKRUSDC","CREAMUSDC",
        "GRTUSDC","ALPHAUSDC","FTTUSDC"
    ],
    "testnet":[
        "ETHUSDC","BTCUSDC","SNXUSDC",
        "LINKUSDC","SDEFIUSDC","TRXUSDC",
        "SCEXUSDC","SUSHIUSDC","XAGUSDC",
        "AUDUSDC","PERPUSDC","UNIUSDC",
        "CRVUSDC","MKRUSDC","CREAMUSDC",
        "GRTUSDC","ALPHAUSDX","FTTUSDC"
    ]
}

DEFAULT_DECIMALS = 18

class Side(Enum):
    LONG = 0
    SHORT = 1

class PnlCalcOption(Enum):
    SPOT_PRICE = 0
    TWAP = 1
    ORACLE = 2




import requests
import json

class MetaData:
    def __init__(self, testnet: bool):
        if testnet:
            self._store =  "https://metadata.perp.exchange/staging.json"
        else:
            self._store = "https://metadata.perp.exchange/production.json"
        self.fetchMeta()

    @property
    def metadataStore(self):
        return self._store

    @metadataStore.setter
    def metadataStore(self, s):
        self._store = s
        self.fetchMeta()
    
    def fetchMeta(self):
        self.meta = requests.get(self._store).json()

    def getL1ContractAddress(self, name):
        return self.meta["layers"]["layer1"]["contracts"][name]["address"]

    def getL2ContractAddress(self, name):
        return self.meta["layers"]["layer2"]["contracts"][name]["address"]

    def getL1ExtContractAddress(self, name):
        return self.meta["layers"]["layer1"]["externalContracts"][name]

    def getL2ExtContractAddress(self, name):
        return self.meta["layers"]["layer2"]["externalContracts"][name]



from setuptools import setup


with open('./README.md') as readme:
    long_description = readme.read()


setup(
    name = 'pyperp',        
    packages = ['pyperp'],  
    version = '0.1.1',      
    license='MIT',        
    description = 'python SDK for Perpetual Protocol',
    long_description_content_type='text/markdown',
    long_description=long_description,
    author = 'Naveen Veluswamy',                   
    author_email = 'velnaveen99@gmail.com',      
    url = 'https://github.com/DeveloperInProgress/PyPerp',   
    keywords = ['perp', 'perpetual protocol', 'defi'],   
    install_requires=[            
        'web3',
        'datetime'
    ],

    include_package_data = True,

    classifiers=[
        'Development Status :: 4 - Beta',      
        'Intended Audience :: Developers',      
        'Topic :: Software Development :: Build Tools',
        'License :: OSI Approved :: MIT License',   
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: 3.9',
    ],
)
